/*
 * Copyright (C) 2012 Chris Burdess <dog@gnu.org>
 *
 * Gnome Shell Biff is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * Gnome Shell Biff is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with Gnome Documents; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

let src = new Array(
    0x41, 0x42, 0x43, 0x44, 0x45, 0x46, 0x47, 0x48, 0x49, 0x4a,
    0x4b, 0x4c, 0x4d, 0x4e, 0x4f, 0x50, 0x51, 0x52, 0x53, 0x54,
    0x55, 0x56, 0x57, 0x58, 0x59, 0x5a, 0x61, 0x62, 0x63, 0x64,
    0x65, 0x66, 0x67, 0x68, 0x69, 0x6a, 0x6b, 0x6c, 0x6d, 0x6e,
    0x6f, 0x70, 0x71, 0x72, 0x73, 0x74, 0x75, 0x76, 0x77, 0x78,
    0x79, 0x7a, 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37,
    0x38, 0x39, 0x2b, 0x2f
);

let dst = [];
for (let i = 0; i < 0xff; i++) {
    dst[i] = -1;
}
for (let i = 0; i < src.length; i++) {
    dst[src[i]] = i;
}

function encode(sb) {
    let target = [];
    let tlen = ((sb.length + 2 - ((sb.length + 2) % 3)) * 4) / 3;
    for (let si = 0; si < sb.length; si += 3) {
        let blen = sb.length - si;
        let i = 0;
        if (blen == 1) {
            let b1 = sb.charCodeAt(si);
            target.push(src[b1 >>> 2 & 0x3f]);
            target.push(src[(b1 << 4 & 0x30) + (i >>> 4 & 0xf)]);
        } else if (blen == 2) {
            let b1 = sb.charCodeAt(si), b2 = sb.charCodeAt(si + 1);
            target.push(src[b1 >>> 2 & 0x3f]);
            target.push(src[(b1 << 4 & 0x30) + (b2 >>> 4 & 0xf)]);
            target.push(src[(b2 << 2 & 0x3c) + (i >>> 6 & 0x3)]);
        } else {
            let b1 = sb.charCodeAt(si), b2 = sb.charCodeAt(si + 1), b3 = sb.charCodeAt(si + 2);
            target.push(src[b1 >>> 2 & 0x3f]);
            target.push(src[(b1 << 4 & 0x30) + (b2 >>> 4 & 0xf)]);
            target.push(src[(b2 << 2 & 0x3c) + (b3 >>> 6 & 0x3)]);
            target.push(src[b3 & 0x3f]);
        }
    }
    while (target.length < tlen) {
        target.push(0x3d);
    }
    let string = "";
    for (let i = 0; i < target.length; i++) {
        string += String.fromCharCode(target[i]);
    }
    return string;
}

function decode(string) {
    let padding = 0;
    while (string.length - padding > 0 && string[string.length - padding - 1] === "=") {
        padding++;
    }
    let slen = string.length - padding;
    let array = [];
    let si = 0, len = slen;
    while (len > 0) {
        let b0 = dst[string.charCodeAt(si++) & 0xff];
        let b2 = dst[string.charCodeAt(si++) & 0xff];
        array.push(b0 << 2 & 0xfc | b2 >>> 4 & 0x3);
        if (len > 2) {
            b0 = b2;
            b2 = dst[string.charCodeAt(si++) & 0xff];
            array.push(b0 << 4 & 0xf0 | b2 >>> 2 & 0xf);
            if (len > 3) {
                b0 = b2;
                b2 = dst[string.charCodeAt(si++) & 0xff];
                array.push(b0 << 6 & 0xc0 | b2 & 0x3f);
            }
        }
        len = slen - si;
    }
    let ret = "";
    for (let i = 0; i < array.length; i++) {
        ret += String.fromCharCode(array[i]);
    }
    return ret;
}

